function z = tremolo(deviation, startfreq, stopfreq, mode)

% define the step size for frequency vector
% adder is the step for arithmetic series (mode = 1)
% multiplr is the step for geometric series (mode = 2)
adder = (10-1)/deviation;
multiplr = 10^(1/deviation);

% determine the exponent k for the highest desired frequency
k = 0;
while 10^k < stopfreq
k = k + 1;
end; % while

% a buffer where the component values 
% are read from a parameter file
RCLvalues = [];
PLOTBUF = [];

% open parameter file, read it line by line,
% and store values to RCLvalues buffer
% by default, the parameter file should be named as params.txt
fid = fopen('params.txt', 'r');
tline = fgetl(fid);
disp(tline);
while 1
tline = fgetl(fid);
if ( ischar(tline) && length(tline) > 1 )
disp(tline);
A = sscanf(tline, '%g');
RCLvalues = [RCLvalues A];
else
break;
endif;
end;
RCLvalues
fclose(fid);

% The outmost loop is determined by 
% the amount of rows in the parameter file.
% This way it is possible to vary component values
% and compare results.
for varied = 1:size(RCLvalues,2)

% read component values into variables
VCC = RCLvalues(1,varied);
VBE = RCLvalues(2,varied);
R2  = RCLvalues(3,varied);
R3  = RCLvalues(4,varied);
R4  = RCLvalues(5,varied);
R5  = RCLvalues(6,varied);
R6  = RCLvalues(7,varied);
R7  = RCLvalues(8,varied);
RS  = RCLvalues(9,varied);
C1  = RCLvalues(10,varied);
C5  = RCLvalues(11,varied);
C2  = RCLvalues(12,varied);
B1  = RCLvalues(13,varied);
RJ  = RCLvalues(14,varied);

RE = R5 + R6;

% calculate the quiescent values to define 
% transistor parameters gm and rpi1
IBQ = (VCC*R3/(R2 + R3) - VBE)/(R2*R3/(R2+R3) + (B1 + 1)*RE)
ICQ = B1*IBQ
V5 = VCC - R4*ICQ
gm = ICQ/0.026
rpi1 = B1/gm

% define empty arrays for the transfer function H and frequency X
H=[];
X=[];
% start from the frequency of 1 Hz
f = 1;

% loop through all the frequency decades 10, 100, 1000, etc.
% according to the exponent k
for j = 0:k
% loop through the specific decade, e.g. 10, 20, 30, 40, ... 90.
while f <= (10^(j+1) - adder)

if (f >= (startfreq - 0.00001) && f <= stopfreq)

% frequency to angular frequency conversion
w = f*2*pi;

% define the admittance matrix elements from the small-signal model
Y11 = 1/RS + i*w*C1;
Y12 = Y21 = i*w*C1;
Y22 = 1/rpi1 + 1/R2 + 1/R3 + i*w*C1;
Y23 = Y32 = 1/rpi1;
Y33 = 1/rpi1 + 1/R5;
Y34 = Y43 = 1/R5;
Y44 = 1/R5 + 1/R6 + (i*w*C5)/(1 + i*w*C5*RJ);
Y55 = 1/R4 + i*w*C2;
Y56 = Y65 = i*w*C2;
Y66 = 1/R7 + i*w*C2;

Y32x = -(B1+1)/rpi1;
Y33x = (B1+1)/rpi1 + 1/R5;
Y52x =  B1/rpi1;
Y53x = -B1/rpi1;

% the numerator matrix (determinant) according to Cramer's rule
os=[ Y11 -Y12    0     0     0      (1/RS); ...
    -Y21  Y22  -Y23    0     0      0; ...
      0   Y32x  Y33x -Y34    0      0; ...
      0    0   -Y43   Y44    0      0; ...
      0   Y52x  Y53x   0    Y55     0; ...
      0    0     0     0   -Y65     0];

% the denominator matrix (determinant) according to Cramer's rule
nim=[ Y11 -Y12    0     0     0      0; ...
     -Y21  Y22  -Y23    0     0      0; ...
       0   Y32x  Y33x -Y34    0      0; ...
       0    0   -Y43   Y44    0      0; ...
       0   Y52x  Y53x   0    Y55   -Y56; ...
       0    0     0     0   -Y65    Y66];
 

% store the transfer function values to the H array
% and the frequency values to the X array
H = [H 20*log10(abs(det(os)/det(nim)))];
X = [X f]; 
endif;

% advance frequency according to the chosen mode
% (arithmetic) or (geometric)
if (mode == 1)
f = f + adder*10^j;
else
f = f*multiplr;
endif;

end; % while
end; % for
PLOTBUF = [PLOTBUF X' H'];
end; % for varied

% save the data for plotting later with gnuplot
% ngspice generates similar data file, 
% so comparison and plotting against that is simple.
save -ascii tremolo_octa.data PLOTBUF

% handle the Octave specific plotting here
figure (2)
semilogx(X, PLOTBUF(:,2:2:end),'LineWidth',3);
set(gca,'FontSize',14)
set(gca,'YLim',[10*(floor(min(min(PLOTBUF(:,2:2:end)))/10)),10*(ceil(max(max(PLOTBUF(:,2:2:end)))/10) + 1)]);
set(gca,'YTick',[10*(floor(min(min(PLOTBUF(:,2:2:end)))/10) - 1):10:10*(ceil(max(max(PLOTBUF(:,2:2:end)))/10) + 1)]);
legend(num2str(PLOTBUF(1,2:2:end)'))
xlabel('frequency [Hz]');
ylabel('magnitude of voltage ratio [dB]');
grid on;
grid minor on;

